/***************************************************************************
	eXynth GUI library
	(C) 2010 Anti Sullin / Artec Design <anti.sullin@artecdesign.ee>

	Based on xynth library:
    (C) 2003 - 2008 by Alper Akcan <distchx@yahoo.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/


#ifndef EVENT_H_
#define EVENT_H_

#include <linux/input.h>
#include "window.h"
#include "timer.h"

#if defined(__cplusplus)
	extern "C" {
#endif

//----------------------------------------------
// Event types:
//  low 16 bits type, upper 16 bits event-specific
#define		EV_CODE			0x0000FFFF
//----------------------------------------------
#define		CREATED_EVENT	0x00000001
#define		CLOSED_EVENT	0x00000002
#define		CHCLOSED_EVENT	0x00000004
#define		MONITOR_EVENT	0x00000008

#define		KEYBD_EVENT		0x00000010
#define		MOUSE_EVENT		0x00000020
#define		TIMER_EVENT		0x00000040

#define		SYSTEM_EVENT	0x00000100
#define		USER_EVENT		0x00000200
//----------------------------------------------
#define		KEYBD_PRESSED	0x10000000
#define		KEYBD_RELEASED	0x20000000
//----------------------------------------------
#define		MOUSE_ENTERED	0x10000000
#define		MOUSE_EXITED	0x20000000
#define		MOUSE_PRESSED	0x40000000
#define		MOUSE_RELEASED	0x80000000
#define		MOUSE_OVER		0x01000000
#define		MOUSE_HINT		0x02000000

#define		MOUSE_LEFTBUTTON	1

//----------------------------------------------
typedef struct s_ev_keyb_s {
	int keycode;

	int handled;
} s_ev_keyb_t;

typedef struct s_ev_mouse_s {
	int x;
	int y;
	int buttons;
	int press;
	long long time;
} s_ev_mouse_t;

typedef struct s_ev_user_s {
	int code;
	void *data;
} s_ev_user_t;

//----------------------------------------------
typedef union s_eventdata_u {
	s_ev_keyb_t		keyboard; // EV_KEYBOARD
	s_ev_mouse_t	mouse; // EV_MOUSE
	s_timer_t		*timer; // EV_TIMER
	int				idwnd; // EV_CREATED, EV_MONITOR
	s_window_t		*child; // EV_CHCLOSED
	s_ev_user_t		user;	// USER_EVENT, SYSTEM_EVENT
} s_eventdata_t;

//----------------------------------------------
typedef struct s_event_s {
	int				type;
	s_eventdata_t	data;
} s_event_t;


/* For backward-compability, define S_ keycodes */
#define S_KEYCODE_ZERO KEY_0
/* KEY1 ... KEY9 id's are in progressing order */
#define S_KEYCODE_ONE KEY_1
#define S_KEYCODE_TWO KEY_2
#define S_KEYCODE_THREE KEY_3
#define S_KEYCODE_FOUR KEY_4
#define S_KEYCODE_FIVE KEY_5
#define S_KEYCODE_SIX KEY_6
#define S_KEYCODE_SEVEN KEY_7
#define S_KEYCODE_EIGHT KEY_8
#define S_KEYCODE_NINE KEY_9
#define S_KEYCODE_DELETE KEY_BACKSPACE
#define S_KEYCODE_RETURN KEY_ENTER
#define S_KEYCODE_PERIOD KEY_DOT
#define S_KEYCODE_UP KEY_UP
#define S_KEYCODE_DOWN KEY_DOWN
#define S_KEYCODE_LEFT KEY_LEFT
#define S_KEYCODE_RIGHT KEY_RIGHT
#define S_KEYCODE_ESCAPE KEY_POWER
#define S_KEYCODE_F1 KEY_F1
#define S_KEYCODE_F2 KEY_F2
#define S_KEYCODE_F3 KEY_F3
#define S_KEYCODE_F4 KEY_F4
#define S_KEYCODE_F5 KEY_F5
#define S_KEYCODE_F11 KEY_F11
#define S_KEYCODE_F12 KEY_MENU
#define S_KEYCODE_F13 KEY_F13
#define S_KEYCODE_F14 KEY_F14

#if defined(__cplusplus)
	}
#endif

#endif /* EVENT_H_ */
