/***************************************************************************
	eXynth GUI library
	(C) 2010 Anti Sullin / Artec Design <anti.sullin@artecdesign.ee>

	Based on xynth library:
    (C) 2003 - 2008 by Alper Akcan <distchx@yahoo.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/


#ifndef EXYNTH_H_
#define EXYNTH_H_

// platform settings
// (FB_ROTATE, COLOR_ORDER_BGR, COL555i, FONT_PATH, FB_DEVICE_NAME)
#include "../include/platform.h"

#include <pthread.h>

#include "surface.h"
#include "timer.h"

#if defined(__cplusplus)
	extern "C" {
#endif

// maximum number of windows (including black background!)
#define S_CLIENTS_MAX	31
#define S_CLIENTS_MAXB	((S_CLIENTS_MAX-1)/8+1)
#define S_MATRIX_FREE	0

#ifndef FONT_PATH
#define FONT_PATH		"/usr/share/fonts"
#endif

#ifndef FB_DEVICE_NAME
#define FB_DEVICE_NAME	"/dev/fb0"
#endif

// global exynth structure
typedef struct s_exynth_s {
	// mutex. Always taken when UI thread is doing work.
	// Lock this mutex when doing some work with UI in other threads.
	// Lock it early (before locking your own mutexes) to avoid deadlocks.
	pthread_mutex_t	mutex;

	pthread_t		thread; 				// input thread
	int				exit;					// input thread exit flag

	s_list_t		timer_list;				// active timers

	s_surface_t		*root_surface;			// root surface with output image
	s_surface_t		*fb_surface;			// framebuffer in case it needs rotating

	s_window_t		*clients[S_CLIENTS_MAX];// client windows [id]
	unsigned char	pri[S_CLIENTS_MAX];		// window stack, 0-bottom, max-top
	unsigned char	pri_top;				// topmost window id

	s_list_t		exitlist;				// queue of windows to be destroyed

	int				mouse_xyid;				// window id at mouse pointer location
	int				mouse_xyid_oldclick;	// window id at last mouse click location

	int				selfpipe[2];			// self-pipe for waking up handler

	int 			(*idle_func)(void);		// callback for power management handler;
											// called on user input to reset idle counters;
											// return 1 to suppress the event
} s_exynth_t;

int s_exynth_init(void);
int s_exynth_uninit(void);

int s_exynth_set_idle_func(int(*f)(void));

int s_exynth_lock(void);
int s_exynth_timedlock(const struct timespec *abs_timeout);
int s_exynth_unlock(void);

s_exynth_t* get_exynth(void);

#if defined(__cplusplus)
	}
#endif

#endif /* EXYNTH_H_ */
