/***************************************************************************
	eXynth GUI library
	(C) 2010 Anti Sullin / Artec Design <anti.sullin@artecdesign.ee>

	Based on xynth library:
    (C) 2003 - 2008 by Alper Akcan <distchx@yahoo.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifndef SURFACE_H_
#define SURFACE_H_

typedef struct s_surface_s s_surface_t;

#include "window.h"
#include "rect.h"

#if defined(__cplusplus)
	extern "C" {
#endif

/** surface modes
  */
typedef enum {
	/** graphics will be drawn on real buffer (->linear_buf) */
	SURFACE_REAL       = 0x1,
	/** graphics will be drawn on virtual buffer (->vbuf) */
	SURFACE_VIRTUAL    = 0x2,
} S_SURFACE_MODE;

/** surface struct
  */
struct s_surface_s {
	/** bitwise ored SURFACE_REAL, SURFACE_VIRTUAL */
	S_SURFACE_MODE mode;
	/** surface buytes per pixel */
	unsigned int bytesperpixel;
	/** surface bits per pixel */
	unsigned int bitsperpixel;
	/** number of colors */
	unsigned int colors;
	/** blue color offset */
	unsigned int blueoffset;
	/** green color offset */
	unsigned int greenoffset;
	/** red color offset */
	unsigned int redoffset;
	/** blue color length */
	unsigned int bluelength;
	/** green color length */
	unsigned int greenlength;
	/** red color length */
	unsigned int redlength;
	/** blue color mask */
	unsigned int bluemask;
	/** green color mask */
	unsigned int greenmask;
	/** red color mask */
	unsigned int redmask;
	/** surface virtual buffer width */
	int width;
	/** surface virtual buffer height */
	int height;
	/** surface's virtual buffer */
	unsigned char *vbuf;
	/** virtual buffers' visible part on screen */
	s_rect_t buf;
	/** memory mapped shared buffer, this is the real video buffer (usually) */
	unsigned char *linear_buf;
	/** video buffer width */
	int linear_buf_width;
	/** video buffer pitch */
	int linear_buf_pitch;
	/** video buffer height */
	int linear_buf_height;
	/** video buffer  size */
	unsigned int linear_mem_size;
	/** shared memory buffer for screen matrix */
	unsigned char *matrix;
};

int s_surface_init (s_window_t *window);
int s_server_surface_init (void);
int s_surface_create (s_window_t *window);
int s_server_surface_create (void);
void s_surface_uninit (s_window_t *window);
int s_server_surface_uninit (void);
void s_surface_defparams (s_surface_t *dst, s_surface_t *src);

// expose
void s_surface_changed (s_window_t *window, s_rect_t *changed, int all);
void s_server_surface_changed (s_rect_t *changed, int all);

#if defined(__cplusplus)
	}
#endif

#endif /* SURFACE_H_ */
