/***************************************************************************
	eXynth GUI library
	(C) 2010 Anti Sullin / Artec Design <anti.sullin@artecdesign.ee>

	Based on xynth library:
    (C) 2003 - 2008 by Alper Akcan <distchx@yahoo.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifndef WINDOW_H_
#define WINDOW_H_

typedef struct s_window_s s_window_t;

#include <pthread.h>
#include "surface.h"
#include "list.h"
#include "event.h"

#if defined(__cplusplus)
	extern "C" {
#endif

/** window types
  */
typedef enum {
	/** window main */
	WINDOW_MAIN       = 0x0001,
	/** window child */
	WINDOW_CHILD      = 0x0002,
	/** window modal child */
	WINDOW_MODAL      = 0x0004,
	/** bitwise or`ed window types */
	WINDOW_TYPES      = WINDOW_MAIN | WINDOW_CHILD | WINDOW_MODAL,
	/** window input, this is a bitwise or`ed property
	  * this type of window will receive all input events,
	  * even if not shown */
	WINDOW_INPUT      = 0x8000,
	/** Monitor window will be notified of window stack changes */
	WINDOW_MONITOR    = 0x4000
} S_WINDOW;

struct s_window_s {
	/** bitwise or'ed window type */
	S_WINDOW type;
	/** parent window */
	s_window_t *parent;

	/** window always ontop property, 0: normal, 1: ontop, -1:on bottom */
	int alwaysontop;
	/** is mouse over */
	int mouse_entered;

	/** surface attached to the window */
	s_surface_t *surface;

	/** atevent event callback */
	void (*atevent) (s_window_t *, s_event_t *);

	/** user data */
	void *data;

	/* surface map id*/
	int id;
	/* is window visible (show/hide)*/
	int visible;
	/* priority (exynth->pri[p]) */
	int pri;
	/* number of modal children, if >0 then input is blocked */
	int num_modal;

	/* wait list */
	s_list_t waitlist;
};

typedef struct s_window_wait_s {
	pthread_mutex_t	mut;
	pthread_cond_t	cond;
	int closed;
} s_window_wait_t;

void s_window_hide (s_window_t *window);
void s_window_show (s_window_t *window);
void s_window_show_prepare (s_window_t *window);
void s_window_set_coor (s_window_t *window, int form, int x, int y, int w, int h);
void s_window_set_alwaysontop (s_window_t *window, int alwaysontop);
int s_window_new (s_window_t *window, S_WINDOW type, s_window_t *parent);
int s_window_event(s_window_t *window, s_event_t *event);
void s_window_broadcast_event(s_event_t *event);
int s_window_init (s_window_t **window);
void s_window_uninit (s_window_t *window);
void s_window_quit (s_window_t *window);
void s_window_perform_quit(void);
void s_window_exit (s_window_t *window);
int s_window_wait (s_window_t *window);
void s_window_atevent (s_window_t *window, void (*f) (s_window_t *, s_event_t *));

#if defined(__cplusplus)
	}
#endif

#endif /* WINDOW_H_ */
