/***************************************************************************
	eXynth GUI library
	(C) 2010 Anti Sullin / Artec Design <anti.sullin@artecdesign.ee>

	Based on xynth library:
    (C) 2003 - 2008 by Alper Akcan <distchx@yahoo.com>
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#ifndef XML_H_
#define XML_H_

#include "list.h"

#if defined(__cplusplus)
	extern "C" {
#endif

typedef struct s_xml_node_attr_s s_xml_node_attr_t;
typedef struct s_xml_node_s s_xml_node_t;

/** xml node attributes struct
  */
struct s_xml_node_attr_s {
	/** attr name */
	char *name;
	/** attr value */
	char *value;
	/** internal dontparse flag */
	int dontparse;
};

/** xml node struct
  */
struct s_xml_node_s {
	/** node name */
	char *name;
	/** node value */
	char *value;
	/** node attributes */
	s_list_t attrs;
	/** child nodes */
	s_list_t nodes;
	/** parent node */
	s_xml_node_t *parent;
	/** internal dontparse flag */
	int dontparse;
};

/** @brief return the node for given path, and head node
  *
  * @param *node - head node to start
  * @param *path - the requested path
  * @returns node for given path, or NULL if not found.
  */
s_xml_node_t * s_xml_node_get_path (s_xml_node_t *node, char *path);

/** @brief returns the value for given node
  *
  * @param *node - the node
  * @returns value for given node, or NULL if no value.
  */
char * s_xml_node_get_value (s_xml_node_t *node);

/** @brief return the node value for given path, and head node
  *
  * @param *node - head node to start
  * @param *path - the requested path
  * @returns value for given path, or NULL if not found.
  */
char * s_xml_node_get_path_value (s_xml_node_t *node, char *path);

/** @brief returns the attribute "attr", for given node
  *
  * @param *node - the node
  * @param *attr - requested attribute
  * @return the attribute, or NULL if not found
  */
s_xml_node_attr_t * s_xml_node_get_attr (s_xml_node_t *node, const char *attr);

/** @brief returns the value for given node, and attribute
  *
  * @param *node - the noe
  * @param *attr - the attribute
  * @returns value for given node, or NULL if no value.
  */
char * s_xml_node_get_attr_value (s_xml_node_t *node, const char *attr);

/** @brief dublicates the given node
  *
  * @param *node - node to dublicate
  * @param **dub - dublicated value
  * @returns 0 on success
  */
int s_xml_node_dublicate (s_xml_node_t *node, s_xml_node_t **dub);

/** @brief dublicates the given attribute
  *
  * @param *attr - node to dublicate
  * @param **dub - dublicated value
  * @returns 0 on success
  */
int s_xml_node_attr_dublicate (s_xml_node_attr_t *attr, s_xml_node_attr_t **dub);

/** @brief return the parent of node with the name "name"
  *
  * @param *node - the node
  * @param *name - name of the parent
  * @returns the parent, or NULL if not found
  */
s_xml_node_t * s_xml_node_get_parent (s_xml_node_t *node, char *name);

/** @brief initializes the attribute struct
  *
  * @param **attr - out value
  * @returns 0 on success
  */
int s_xml_node_attr_init (s_xml_node_attr_t **attr, const char *name, const char *val);

/** @brief uninitializes the attribute node
  *
  * @param *attr - the attribute
  * @returns 0 on success
  */
int s_xml_node_attr_uninit (s_xml_node_attr_t *attr);

/** @brief initializes the node struct
  *
  * @param **node - out value
  * @returns 0 on success
  */
int s_xml_node_init (s_xml_node_t **node, const char *name);

/** @brief uninitializes the node
  *
  * @param *node - the node
  * @returns 0 on success
  */
int s_xml_node_uninit (s_xml_node_t *node);

/** @brief parses the given buffer
  *
  * @param **node  - out value
  * @param *buffer - buffer to parse
  * @param len     - buffer length
  * @returns 0 on success
  */
int s_xml_parse_buffer (s_xml_node_t **node, char *buffer, unsigned int len);

/** @brief parses the given file
  *
  * @param **node - out value
  * @param *file  - file to parse
  * @returns 0 on success
  */
int s_xml_parse_file (s_xml_node_t **node, char *file);

/*@}*/

#if defined(__cplusplus)
	}
#endif

#endif /* XML_H_ */
