#ifndef GUIMULTITEXTBOX_H_
#define GUIMULTITEXTBOX_H_

#include <string>
#include <list>

#include "GuiPrimitive.h"
#include "GuiTextWrapper.h"

#include <exynth/exynth.h>
#include <exynth/font.h>

#define GUIMULTITEXTBOX(f,p) getPrimitive<GuiMultiTextBox>(f,p)

/**
 * \brief Multiline text box primitive
 *
 * Implements multiline text box primitive. Multiline text box can hold
 * more than one line of data. This class provides basic management methods
 * for that type of text boxes.
 */
class GuiMultiTextBox : public GuiPrimitive
{
public:
	
	/** Horizontal alignment type */
	enum TextHAlignment
	{
		/** Align left */
		LEFT, 
		/** Align right */
		RIGHT, 
		/** Align center */
		HCENTER 
	};
	
	/** Vertical alignment type */
	enum TextVAlignment
	{
		/** Align top */
		TOP,
		/** Align bottom */
		BOTTOM,
		/** Align middle */
		VCENTER,
	};

	GuiMultiTextBox(int idResource, GuiFrame* pFrame);
	virtual ~GuiMultiTextBox();

	void setFont(const char* fontName, int fontSize);
	void setColor(const char *pColor);
	virtual void setText(const char* boxText);
	void setAlignment(TextHAlignment horz, TextVAlignment vert = TOP);
	
	void getRequiredSize(s_rect_t &rect);
	
protected:
	void renderText();

	virtual void onPaint(GuiSurface *drawSurface, s_rect_t &paintArea);
	
	/** Instance of the text wrapper helper class */
	GuiTextWrapper	wrapper;

	/** Line height of the text box */
	int				lineHeight;

	/** Actual text height */
	int				textHeight;

	/** Set to true if the text is rendered already, false otherwise */
	bool			isRendered;

	/** Pointer to the text the text box should contain. Used only to calculate character positions in actual string */
	const char		*userText; // used only to calculate char positions in string

	/** List of lines the text box contains */
	std::list<textLine> *lineList;

	/** List to hold the font information */
	std::list<s_font_t*> fontData;
	
	/** Name of the font to use for rendering */
	std::string		fontName;

	/** Color to use for text rendering */
	RGB_COLOR		fontColor;

	/** Size of the font to use for rendering */
	int				fontSize;

	/** Type of horizontal text alignment */
	TextHAlignment	horzAlign;

	/** Type of vertical text alignment */
	TextVAlignment	vertAlign;
};

#endif /*GUIMULTITEXTBOX_H_*/
