/*
 * GuiTimer.h
 *
 *  Created on: 12.02.2010
 *      Author: anti
 */

#ifndef GUITIMER_H_
#define GUITIMER_H_

#include <exynth/timer.h>

/**
 * \brief GUI timer class
 *
 * Implements a standalone polling timer. Overload this class to create a
 * timer-polling object that is not related to any window. Within windows,
 * use window timers.
 *
 * Timer ID's:
 *  * 0-499:    guiOO internal (effectwindow etc)
 *  * 500-999:  guiOO internal primitives (animation etc)
 *  * 1000+:    user-allocation
 *
 * \author Anti Sullin
 */
class GuiTimer
{
public:
	GuiTimer();
	virtual ~GuiTimer();

	bool startTimer();
	void stopTimer();

protected:
	/**
	 * \brief Execute when the timer fires
	 * Pure virtual method that will be executed when the timer fires
	 * \note This method must be overridden in the class that implements GuiTimer capabilities
	 */
	virtual void onTimer() = 0;

	static void timerProc(s_window_t *window, s_timer_t *timer);

	/** Period after which the timer is fired */
	int			interval;

	/** Pointer to structure where the timer options are stored */
	s_timer_t	*timer;
};

#endif /* GUITIMER_H_ */
