#ifndef BARCODEREADER_H_
#define BARCODEREADER_H_

#include "exynthoo/GuiTimer.h"
#include "Thread.h"
#include "SerialPort.h"
#include "BasicReader.h"

#define BARCODE_MAX_BYTES  63

/**
 * \brief Barcode reader handler
 *
 * Implements methods to handle the barcode reader.
 * The reader will be be initialized by this class, and the appropriate functions
 * can be called to trigger the laser beam and submit the retrieved data.
 */
class BarcodeReader: public BasicReader, protected SerialPort, protected Thread, protected GuiTimer
{
public:
	BarcodeReader();
	virtual ~BarcodeReader();

	virtual bool initialize();
	virtual void shutdown();

	virtual void initRead();
	virtual void startRead();
	virtual void stopRead();

protected:
	virtual void run();
	virtual void onTimer();

	void submit(const char *buf);

private:
	int devHandle;
	bool readerActive;
};

#endif /*BARCODEREADER_H_*/
