/*
 * BlueGigaBluetoothModule.h
 *
 *  Created on: 21.06.2011
 *      Author: ivari
 */

#ifndef BLUEGIGABLUETOOTHMODULE_H_
#define BLUEGIGABLUETOOTHMODULE_H_

#include <string>

#include "Thread.h"

#include "BluetoothModule.h"
#include "BluetoothPort.h"

class BlueGigaBluetoothModule : public BluetoothModule {

public:

	static ModemAnswers answers[];

	BlueGigaBluetoothModule(const char* portName);
	virtual ~BlueGigaBluetoothModule();

	bool open();
	bool close();

	bool lock();
	void unlock();

	virtual bool shutdown();

	virtual std::string getLocalMac();
	virtual std::string getVersion();
	virtual std::vector<Device>scanDevices();

	virtual void setEnabled(bool enabled);
	BluetoothModule::AnswerType getAnswerType();

	bool setLAP(const char* lapID);
	bool enableMUX();
	bool disableMUX();
	bool isAlive();
	bool ping();
	bool disableEcho();
	bool setAuth(char * pin);
	std::string queryActiveConnections();
	int getSignalStrength();
	void pollIncomingData(int timeOut);


protected:
	virtual bool sendSPPData(int connid, const void *data, size_t len);

	virtual bool addSDPRecord(int localPort, void* data, size_t len);

	virtual bool connectSPP(int idConn, unsigned char *dstAddr, unsigned char dstPort);
	virtual bool listenSPP(int idConn);
	virtual bool disconnectSPP(int idConn);


	std::string			commPortName;
	BluetoothPort*		btPort;
	bool				moduleEnabled;
	bool				muxEnabled;

	Mutex				btMutex;

private:
	size_t createMuxFrame(char *buf, size_t buflen, const char *cmd, size_t cmdlen, int channel);
	size_t parseMuxFrame(std::string *buf, char *frame, size_t pos, size_t len);
	size_t setAnswerType(const char *buf, size_t len);

	bool writeCommand(const char* cmd, size_t cmdlen, int channel=0xFF);
	bool readAnswer(std::string *pAnswer, int timeOut);

};

#endif /* BLUEGIGABLUETOOTHMODULE_H_ */
