#ifndef BLUETOOTHSPPCONNECTION_H_
#define BLUETOOTHSPPCONNECTION_H_

#include "Thread.h"
#include "BluetoothModule.h"
#include "SerialPort.h"

#define BT_SPP_FIFO_SIZE	1024

/**
 * \brief Bluetooth SPP Connection class
 *
 * Implements a Bluetooth SPP Connection (Serial Port Protocol, RFComm layer).
 * Every SPPConnection has its Rx FIFO and handles its connection state and data transmission.
 * Use BluetoothModule->createSPPConnection(...) to create it.
 */
class BluetoothSPPConnection : public SerialPortInterface
{
public:
	~BluetoothSPPConnection();

	virtual bool reOpen();
	virtual bool isPortOpen();

	virtual ssize_t writeData(const void *buf, size_t len);
	virtual ssize_t readData(void *buf, size_t buflen);
	virtual size_t numInputPending(int timeout);
	virtual bool isInputPending(int timeOut);

	virtual void flushInput();
	virtual void flushOutput();
	virtual void drainOutput();

	int getFifoCnt();

	bool isConnected();
	void pollData();

	bool getDestAddr(unsigned char *pAddr);
	int getDestPort();
	int getLocalPort();
	void getPin(char *pin, size_t pinlen);

	bool setDestAddr(const unsigned char *destaddr);
	bool setDestPort(int destPort);
	bool setLocalPort(int localPort);
	bool setPin(const char *pin);

	bool connect();
	bool listen();
	bool disconnect();



	friend class BluetoothModule;

protected:
	BluetoothSPPConnection(BluetoothModule *pBtModule, bool incoming);
	void setConnectionState(SPP_CONN_STATUS connStatus);
	void incomingData(const void *buf, size_t len);

	bool registerSPP();

private:
	bool incoming;

	Semaphore bufSem;
	BluetoothModule *pBtModule;

	int connId;
	int channel;
	unsigned char dstAddr[6];
	int dstPort;
	int localPort;
	char pin[MAX_BT_PIN];

	Semaphore		linkStatusSem; // notify of link status change
	SPP_CONN_STATUS connStatus;

	unsigned char fifo[BT_SPP_FIFO_SIZE];
	int fifo_r, fifo_w;
};

#endif /* BLUETOOTHSPPCONNECTION_H_ */
