/*
 * GprsConnection.h
 *
 *  Created on: Nov 19, 2008
 *      Author: andrei
 */

#ifndef GPRSCONNECTION_H_
#define GPRSCONNECTION_H_

#include <string>

#include "NetworkConnection.h"
#include "GsmModule.h"
#include "Gsm0710Mux.h"
#include "GprsPpp.h"

/** Parameters required to connect to a specific GPRS network. */
struct GPRS_CONNECTION_SETTINGS
{
	/** Contains the SIM PIN code */
	std::string					simPin;
	/** Specifies the GSM frequency band */
	GsmModule::ModuleFreqBand	gsmBand;
	/** Contains the GPRS Access Point Name */
	std::string					gprsApn;
	/** Contains the user name for GPRS PPP authorization */
	std::string					authUser;
	/** Contains the password for GPRS PPP authorization */
	std::string					authPass;
};

/**
 * \brief GPRS connection class
 *
 * Implements the GPRS network connection.
 * Currently supported is the Artec GSM/GPRS mini-PCI express card with
 * Telit GE864 modem.
 */
class GprsConnection: public NetworkConnection
{
public:

	GprsConnection(GPRS_CONNECTION_SETTINGS *pSettings);
	virtual ~GprsConnection();

	GsmModule *getGsmModule();

	virtual bool isConnectionUp();
	virtual bool getNetworkInfo(GENERIC_NETWORK_INFO &netInfo);

protected:

	virtual bool netInitialize();
	virtual bool netConnect();
	virtual bool netDisconnect();
	virtual bool netSuspend();
	virtual bool netResume();
	virtual bool netShutdown();

private:

	GsmModule					gsmModule;
	Gsm0710Mux					serialMux;
	GPRS_CONNECTION_SETTINGS	cfg;
};

#endif /* GPRSCONNECTION_H_ */
