#ifndef _GSM0710_BUFFER_H_
#define _GSM0710_BUFFER_H_

#define MAX_BUFFER		4096

// GSM0710 flags
#define EA				0x01
#define CR				0x02
#define	PF				0x10
#define FLAG			0xF9

// the types of the control channel commands (information frames)
// multiplexer disconnect command
#define TYPE_CLD		0xC1
// test link command
#define TYPE_TEST		0x21
// modem status command
#define TYPE_MSC 	  	0xE1
// "not supported" command
#define TYPE_NSC		0x11
// power saving mode command
#define TYPE_PSC		0x41

// V.24 flow control flags
#define V24_FC			0x02
#define V24_RTC			0x04
#define V24_RTR			0x08
#define V24_IC			0x40
#define V24_DV			0x80

// set ansynchronous balanced mode
#define CONTROL_SABM	0x2F
// unnumbered acknowledgement
#define CONTROL_UA		0x63
// disconnected mode
#define CONTROL_DM		0x0F
// disconnect
#define CONTROL_DISC	0x43
// unnumbered information
#define CONTROL_UI		0x03
// unnumbered information with header check
#define CONTROL_UIH		0xEF

/**
 * \brief GSM 07.10 multiplexer frame class
 *
 * Implements the GSM 07.10 multiplexer frame related functionality.
 */
class Gsm0710Frame
{
public:

	Gsm0710Frame();
	~Gsm0710Frame();

	bool matchCmdType(unsigned char type);
	bool matchControl(unsigned char ctl);

	/** Data link connection identifier */
	unsigned char dlci;

	/** Control field that specifies frame type */
	unsigned char control;

	/** Frame data buffer */
	unsigned char *data;

	/** Data buffer length */
	size_t length;
};

/**
 * \brief GSM 07.10 multiplexer buffer class
 *
 * Implements the GSM 07.10 multiplexer buffer related functionality.
 */
class Gsm0710Buffer
{
public:

	Gsm0710Buffer();
	~Gsm0710Buffer();

	size_t getFree();
	size_t getAllocated();

	size_t write(const unsigned char* data, size_t len);
	Gsm0710Frame* read();

	static unsigned char makeCheckSum(const unsigned char *data, size_t len);

private:

	unsigned char *advanceBufPtr(unsigned char *ptr);

	unsigned char* pRead;
	unsigned char* pWrite;

	unsigned char dataBuffer[MAX_BUFFER];
};

#endif /* _GSM0710_BUFFER_H_ */
