#ifndef LMXBluetoothPort_H_
#define LMXBluetoothPort_H_

#include "SerialPort.h"

#define DEFAULT_BT_TIMEOUT			5000

#define MAX_BT_PKT_DATA_LEN    333


/* --------- lmx98xx communication protocol -----------*/

// Packet delimiters
#define STX       ((unsigned char) 0x02)   // start
#define ETX       ((unsigned char) 0x03)    // stop
// Packet types
#define REQ       'R'                       // request
#define CFM       'C'                       // confirm
#define IND       'i'                       // indication
#define RES       'r'                       // response


// lmx98xx packet header format
/** Structure to hold the LMX98xx packet header format */
struct bt_header {
	/** Start delimiter */
	unsigned char stx;
	/** Packet type identification */
	unsigned char pktype;
	/** Opcode (the actual command) */
	unsigned char opcode;
	/** Payload length */
	unsigned short len;
	/** Checksum */
	unsigned char crc;
}__attribute__ ((packed));

/**
 * \brief Bluetooth port class
 *
 * Implements the low-level National LMX-series Bluetooth serial port protocol.
 */
class LMXBluetoothPort: public SerialPort
{
public:

	LMXBluetoothPort();
	LMXBluetoothPort(const char* portName);

	bool sendCmd(unsigned char cmd,
			const void *buf1, size_t len1,
			const void *buf2, size_t len2);

	int getPkt(unsigned char *type, unsigned char *opcode,
			void *buf, size_t buflen, int timeOut);
};

#endif /*LMXBluetoothPort_H_*/
