#ifndef LOGFILE_H_
#define LOGFILE_H_

#include <stdlib.h>
#include <stdarg.h>
#include <string>

/**
 * \brief Log file class
 *
 * Log file class provides facilities to store messages to a text file with simple formatting.
 */
class LogFile
{
public:
	LogFile(const char* fileName);
	virtual ~LogFile();

	/** Log levels */
	enum LogLevel {
		/** Log everything */
		LOG_DEBUG=0,
		/** Log informational messages */
		LOG_INFO,
		/** Log warnings */
		LOG_WARNING,
		/** Log errors only */
		LOG_ERROR,
		/** Log critical errors */
		LOG_CRITICAL
	};

	bool logPrintf(LogLevel logLevel, const char* entryId, const char* entryFmt, ...);
	bool logvPrintf(LogLevel logLevel, const char* entryId, const char* entryFmt, va_list arg);

private:

	std::string fileName;
};

/**
 * \brief Log file class
 *
 * Raw Log file class provides facilities to store messages to a text file without formatting.
 */
class RawLogFile
{
public:
	RawLogFile(const char* fileName);
	virtual ~RawLogFile();

	bool logPrintf(const char* entryFmt, ...);
	bool logvPrintf(const char* entryFmt, va_list arg);

private:

	std::string fileName;
};

#endif /*LOGFILE_H_*/
