#ifndef MEDIA_H_
#define MEDIA_H_

#include "Thread.h"

/** Media is in error state */
#define MEDIA_STATE_ERROR		-1
/** Media is unmounted */
#define MEDIA_STATE_UNMOUNT		0
/** Media is mounted read only */
#define MEDIA_STATE_READONLY	1
/** Media is mounter read-write mode */
#define MEDIA_STATE_READWRITE	2
/** Media does not need mounting */
#define MOUNT_NOT_REQUIRED		0
/** Mounting media failed */
#define MOUNT_FAILED			-1

/** Structure to hold mount point attributes. */
struct MountPoint
{
	/** Mount point location in system root */
	const char	mountPoint[32];
	/** Media device file in system root */
	const char	devName[32];
	/** Filesystem type, e.g. vfat or ext2 */
	const char	fsType[16];
	/** Unmount type, should be set tp either MEDIA_STATE_UNMOUNT
	 * or MEDIA_STATE_READONLY */
	const int	unmountType;
	/** Number of mountpoint readers */
	int num_readers;
	/** Number of mountpoint writers */
	int num_writers;
};

/**
 * \brief Media mounter class
 *
 * Media mounter is a helper class designed to simplify media mounting
 * and unmounting when a corresponding file located on it is being
 * opened or closed. Media mounter has static functions and does not
 * require initialziation.
 */
class Media
{
public:

	static int mountMedia(const char* fileName, bool writeFlag);
	static bool unmountMedia(int mountPoint);
	static int getMediaState(int mountPoint);
	static int getMediaState(const char *fileName);
	static bool unmountAll();
	static void mountEnable();
	static void mountDisable();

private:

	static bool belongsToMountPoint(const char *mountPoint, const char *fileName);

	static MountPoint	mountTable[];
	static Mutex		mountMutex;
	static bool mountEnabled;
};

/** \file */
#endif /*MEDIA_H_*/
