#ifndef SCANNERINPUT_H_
#define SCANNERINPUT_H_

#include "PowerManager.h"
#include "BasicReader.h"
#include "RfidReader.h"

class MainWindow;

//class ScannerInput;
////////////////////////////////////////////////////////////////////////////////
/**
 * \brief Generic scanner input class
 * Generic class to handle various scanner input devices. Provides static initializer
 * method to create the class instance. This instance can be later retrieved by using
 * getScannerInput() method.
 *
 * The class contains all common methods for handling scanner devices.
 */
class ScannerInput
{
public:

	/** List of scanner types */
	enum ScannerType
	{
		/** Barcode scanner */
		SCANNER_BARCODE = 0,
		/** RFID scanner */
		SCANNER_RFID,
		/** Maximum number of available scanner types */
		MAX_READERS
	};

	ScannerInput();
	virtual ~ScannerInput();

	static bool initialize();
	static void shutdown();
	static ScannerInput* getScannerInput();

	bool initScan(int scanner);
	bool startScan();
	bool stopScan();
	void setRfidReadAreas(RfidBank bank, int first_block, int last_block);

protected:

	/** Instance of this class */
	static ScannerInput *scannerInput;

	bool init();
	void close();

private:

	void detectReaders();

	BasicReader*			pReaders[MAX_READERS];
	bool					scanInited[MAX_READERS];
};

#endif /*SCANNERINPUT_H_*/
