#ifndef SYSINFO_H_
#define SYSINFO_H_

#include <platform.h>
#include <string>

#define FLASH_DEV			"/dev/mtd2"
#define FLASH_OFFSET		4
#define MAX_VAR_SIZE		32
#define MAX_ENV_SIZE		1024

#define UBOOT_VAR_SERIAL		"serial"
#define UBOOT_VAR_MODEL			"model"
#define UBOOT_VAR_HWREV			"hwrev"
#define UBOOT_VAR_MFTIMESTAMP	"mfdate"
#define UBOOT_VAR_COMM_TYPE		"cmtype"

/** Information about the communication module of the mobile data terminal. */
struct SCANNER_INFORMATION
{
	/** Contains the reader type identifier. Currently known modules are:
	 * - 000 -- no scanner capabilities
	 * - 100 -- 1D laser reader
	 * - 110 -- 1D laser reader and RFID reader
	 */
	char readerType[8];
	/** Contains the serial number of the communication module */
	char serialNumber[8];
	/** Contains the factory manufacture time stamp, Unix timestamp format */
	unsigned long factoryTimeStamp;
};

/**
 * \brief System information class
 *
 * System information class provides facilities to retrieve system hardware and software
 * information, and global hardware configuration data.
 * System information class contains static functions and does not require separate
 * initialization.
 */
class SysInfo
{
public:

	static const char* getSerialNumber();
	static const char* getModelType();
	static const char* getHardwareRevision();
	static const char* getFirmwareVersion();
	static time_t getManufTimeStamp();
	static SCANNER_INFORMATION* getScannerInformation();
	static int getCommModule();

	static bool getUbootEnv(const char* varName, char* pBuf, int maxLen);

private:

	static char serialNumber[MAX_VAR_SIZE];
	static char modelType[MAX_VAR_SIZE];
	static time_t manufTimeStamp;
	static char hardwareRevision[MAX_VAR_SIZE];
	static char firmwareVersion[MAX_VAR_SIZE];
	static char envBuffer[MAX_ENV_SIZE];
	static SCANNER_INFORMATION scannerInfo;
	static unsigned char scannerInfoVersion;
	static int commModuleType;
};

#endif /*SYSINFO_H_*/
